// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>
#include "sim.h"
#include <math.h>

void save_config(char *simfile,struct device *in)
{
int i;
char name[1000];
FILE *config;

config=fopen(simfile,"w");
if (config == NULL)
{
	printf("Can not write file to disk %s\n",name);
	exit(0);
}

fprintf(config,"#bdos\n");
fprintf(config,"%d\n",in->bdos);
fprintf(config,"#enableexinnewton\n");
fprintf(config,"%d\n",in->enable_ex);
fprintf(config,"#nnewton\n");
fprintf(config,"%d\n",in->nnewton);
fprintf(config,"#hnewton\n");
fprintf(config,"%d\n",in->hnewton);
fprintf(config,"#lnewton\n");
fprintf(config,"%d\n",in->lnewton);
fprintf(config,"#ntrapnewton\n");
fprintf(config,"%d\n",in->ntrapnewton);
fprintf(config,"#ptrapnewton\n");
fprintf(config,"%d\n",in->ptrapnewton);
fprintf(config,"#combitrap\n");
fprintf(config,"%d\n",in->combitrap);
fprintf(config,"#mesh\n");
fprintf(config,"%d\n",in->ymeshlayers);


for (i=0;i<in->ymeshlayers;i++)
{
	fprintf(config,"%le %lf\n",in->meshdata[i].len,in->meshdata[i].number);
}



fprintf(config,"%d\n",in->mat.number);

for (i=0;i<in->mat.number;i++)
{
	fprintf(config,"%s %le %le %le\n",in->mat.l[i].name,in->mat.l[i].height,in->mat.l[i].Nad,in->mat.l[i].space);
}

fprintf(config,"#lcontact\n");
fprintf(config,"%s\n",in->lcontact.name);

fprintf(config,"#rcontact\n");
fprintf(config,"%s\n",in->rcontact.name);

fprintf(config,"#xlen\n");
fprintf(config,"%le\n",in->xlen);
fprintf(config,"#zlen\n");
fprintf(config,"%le\n",in->zlen);
fprintf(config,"#plot\n");
fprintf(config,"%d\n",in->enable_plot);
fprintf(config,"#maxelectricalitt\n");
fprintf(config,"%d\n",in->max_electrical_itt);
fprintf(config,"#electricalclamp\n");
fprintf(config,"%lf\n",in->electrical_clamp);
fprintf(config,"#posclamp\n");
fprintf(config,"%lf\n",in->posclamp);
fprintf(config,"#thermalclamp\n");
fprintf(config,"%lf\n",in->thermal_clamp);
fprintf(config,"#enableback\n");
fprintf(config,"%d\n",in->enable_back);
fprintf(config,"#electricalerror\n");
fprintf(config,"%le\n",in->min_cur_error);
fprintf(config,"#Psun\n");
fprintf(config,"%le\n",in->Psun);
fprintf(config,"#NDfilter\n");
fprintf(config,"%le\n",in->ND);
fprintf(config,"#Vstart\n");
fprintf(config,"%le\n",in->Vstart);
fprintf(config,"#Vstop\n");
fprintf(config,"%le\n",in->Vstop);
fprintf(config,"#Vstep\n");
fprintf(config,"%le\n",in->Vstep);
fprintf(config,"#Rbi\n");
fprintf(config,"%d\n",in->Rbi_on);
fprintf(config,"#Gp\n");
fprintf(config,"%d\n",in->Gp_on);
fprintf(config,"#Gn\n");
fprintf(config,"%d\n",in->Gn_on);
fprintf(config,"#excitonexternal\n");
fprintf(config,"%d\n",in->external_exciton_solver);
fprintf(config,"#thermal\n");
fprintf(config,"%d\n",in->thermal_on);
fprintf(config,"#Tll\n");
fprintf(config,"%lf\n",in->Tll);
fprintf(config,"#Tlr\n");
fprintf(config,"%lf\n",in->Tlr);
fprintf(config,"#Tliso\n");
fprintf(config,"%d\n",in->Tliso);
fprintf(config,"#timeon\n");
fprintf(config,"%d\n",in->time_on);
fprintf(config,"#doce\n");
fprintf(config,"%d\n",in->do_ce);
fprintf(config,"#testswitch\n");
fprintf(config,"%d\n",in->testswitch);

fprintf(config,"#dt\n");
fprintf(config,"%le\n",in->dt);
fprintf(config,"#tsteps\n");
fprintf(config,"%d\n",in->tsteps);
fprintf(config,"#pulsestart\n");
fprintf(config,"%d\n",in->pulsestart);
fprintf(config,"#pulselen\n");
fprintf(config,"%d\n",in->pulselen);
fprintf(config,"#reprate\n");
fprintf(config,"%d\n",in->reprate);


fprintf(config,"#onvalue\n");
fprintf(config,"%lf\n",in->onvalue);
fprintf(config,"#offvalue\n");
fprintf(config,"%lf\n",in->offvalue);
fprintf(config,"#pulsevoltage\n");
fprintf(config,"%d\n",in->pulsevoltage);

fprintf(config,"#tau_e\n");
fprintf(config,"%e\n",in->tau_e);
fprintf(config,"#tau_h\n");
fprintf(config,"%e\n",in->tau_h);
fprintf(config,"#jouleheat\n");
fprintf(config,"%d\n",in->jouleheat);

fprintf(config,"#Rshunt\n");
fprintf(config,"%le\n",in->Rshunt);
fprintf(config,"#Rcontact\n");
fprintf(config,"%le\n",in->Rcontact);
in->Rcontact/=100.0;
in->Rcontact/=100.0;
fprintf(config,"#Dphoton\n");
fprintf(config,"%d\n",in->Dphoton);
fprintf(config,"#Dphotoneff\n");
fprintf(config,"%lf\n",in->Dphotoneff);
fprintf(config,"#Spectra\n");
fprintf(config,"%s\n",in->spectra);
fprintf(config,"#Vbias\n");
fprintf(config,"%lf\n",in->Vbias);
fprintf(config,"#Rbitype\n");
fprintf(config,"%d\n",in->Rbitype);

fclose(config);

}

void load_config(char *simfile,struct device *in,int gui)
{

int i;
char name[1000];
FILE *config;


config=fopen(simfile,"r");
if (config == NULL)
{
	printf("no file %s\n",name);
	exit(0);
}
fscanf(config,"%s",name);
fscanf(config,"%d",&(in->bdos));
fscanf(config,"%s",name);
fscanf(config,"%d",&(in->enable_ex));
fscanf(config,"%s",name);
fscanf(config,"%d",&(in->nnewton));
fscanf(config,"%s",name);
fscanf(config,"%d",&(in->hnewton));
fscanf(config,"%s",name);
fscanf(config,"%d",&(in->lnewton));
fscanf(config,"%s",name);
fscanf(config,"%d",&(in->ntrapnewton));
fscanf(config,"%s",name);
fscanf(config,"%d",&(in->ptrapnewton));
fscanf(config,"%s",name);
fscanf(config,"%d",&(in->combitrap));
fscanf(config,"%s",name);
fscanf(config,"%d",&(in->ymeshlayers));


in->meshdata = malloc (in->ymeshlayers * sizeof(struct mesh));

in->ymeshpoints=0;

for (i=0;i<in->ymeshlayers;i++)
{
	fscanf(config,"%lf %lf",&(in->meshdata[i].len),&(in->meshdata[i].number));
	in->meshdata[i].den=in->meshdata[i].len/in->meshdata[i].number;
	in->ymeshpoints+=in->meshdata[i].number;
}

in->ymatlayers=malloc (in->ymeshpoints * sizeof(int));

if (gui==FALSE)
{
	in->ymesh = malloc (in->ymeshpoints * sizeof(double));

	int pos=0;
	int ii;
	double dpos=0.0;
	for (i=0;i<in->ymeshlayers;i++)
	{

		for (ii=0;ii<in->meshdata[i].number;ii++)
		{
			dpos+=in->meshdata[i].den/2.0;
			in->ymesh[pos]=dpos;
			in->ymatlayers[pos]=i;
			dpos+=in->meshdata[i].den/2.0;
			pos++;
		}
	}
}

fscanf(config,"%d",&(in->mat.number));
in->mat.l = malloc (in->mat.number * sizeof(struct layer));

for (i=0;i<in->mat.number;i++)
{
	fscanf(config,"%s %le %le %le",in->mat.l[i].name,&(in->mat.l[i].height),&(in->mat.l[i].Nad),&(in->mat.l[i].space));
	if (gui==FALSE) load_mat_file(in->mat.l[i].name,&(in->mat.l[i]));

}

fscanf(config,"%s",name);
fscanf(config,"%s",in->lcontact.name);
//printf("%s\n",in->lcontact.name);
//getchar();

if (gui==FALSE) load_mat_file(in->lcontact.name,&(in->lcontact));

fscanf(config,"%s",name);
fscanf(config,"%s",in->rcontact.name);
if (gui==FALSE) load_mat_file(in->rcontact.name,&(in->rcontact));

fscanf(config,"%s",name);
fscanf(config,"%le",&(in->xlen));
fscanf(config,"%s",name);
fscanf(config,"%le",&(in->zlen));
fscanf(config,"%s",name);
fscanf(config,"%d",&(in->enable_plot));
fscanf(config,"%s",name);
fscanf(config,"%d",&(in->max_electrical_itt));
fscanf(config,"%s",name);
fscanf(config,"%lf",&(in->electrical_clamp));
//printf("%s\n",name);
//getchar();

fscanf(config,"%s",name);
fscanf(config,"%lf",&(in->posclamp));
//printf("%s\n",name);
//getchar();

fscanf(config,"%s",name);
fscanf(config,"%lf",&(in->thermal_clamp));
//printf("%s\n",name);
//getchar();

fscanf(config,"%s",name);
fscanf(config,"%d",&(in->enable_back));
//printf("%s\n",name);
//getchar();

fscanf(config,"%s",name);
fscanf(config,"%le",&(in->min_cur_error));
//printf("%s %le\n",name,in->min_cur_error);
//getchar();
fscanf(config,"%s",name);
fscanf(config,"%le",&(in->Psun));
fscanf(config,"%s",name);
fscanf(config,"%le",&(in->ND));
fscanf(config,"%s",name);
fscanf(config,"%le",&(in->Vstart));
fscanf(config,"%s",name);
fscanf(config,"%le",&(in->Vstop));
fscanf(config,"%s",name);
fscanf(config,"%le",&(in->Vstep));
fscanf(config,"%s",name);
fscanf(config,"%d",&(in->Rbi_on));
fscanf(config,"%s",name);
fscanf(config,"%d",&(in->Gp_on));
fscanf(config,"%s",name);
fscanf(config,"%d",&(in->Gn_on));
fscanf(config,"%s",name);
fscanf(config,"%d",&(in->external_exciton_solver));
fscanf(config,"%s",name);
fscanf(config,"%d",&(in->thermal_on));
fscanf(config,"%s",name);
fscanf(config,"%lf",&(in->Tll));
fscanf(config,"%s",name);
fscanf(config,"%lf",&(in->Tlr));
fscanf(config,"%s",name);
fscanf(config,"%d",&(in->Tliso));
fscanf(config,"%s",name);
fscanf(config,"%d",&(in->time_on));
fscanf(config,"%s",name);
fscanf(config,"%d",&(in->do_ce));
fscanf(config,"%s",name);
fscanf(config,"%d",&(in->testswitch));

fscanf(config,"%s",name);
fscanf(config,"%lf",&(in->dt));
fscanf(config,"%s",name);
fscanf(config,"%d",&(in->tsteps));
fscanf(config,"%s",name);
fscanf(config,"%d",&(in->pulsestart));
fscanf(config,"%s",name);
fscanf(config,"%d",&(in->pulselen));
fscanf(config,"%s",name);
fscanf(config,"%d",&(in->reprate));


fscanf(config,"%s",name);
fscanf(config,"%lf",&(in->onvalue));
fscanf(config,"%s",name);
fscanf(config,"%lf",&(in->offvalue));
fscanf(config,"%s",name);
fscanf(config,"%d",&(in->pulsevoltage));

fscanf(config,"%s",name);
fscanf(config,"%le",&(in->tau_e));
fscanf(config,"%s",name);
fscanf(config,"%le",&(in->tau_h));
fscanf(config,"%s",name);
fscanf(config,"%d",&(in->jouleheat));


fscanf(config,"%s",name);
fscanf(config,"%le",&(in->Rshunt));
fscanf(config,"%s",name);
fscanf(config,"%le",&(in->Rcontact));
fscanf(config,"%s",name);
fscanf(config,"%d",&(in->Dphoton));
fscanf(config,"%s",name);
fscanf(config,"%lf",&(in->Dphotoneff));
in->Dphotoneff=fabs(in->Dphotoneff);

fscanf(config,"%s",name);
fscanf(config,"%s",in->spectra);
fscanf(config,"%s",name);
fscanf(config,"%lf",&(in->Vbias));
fscanf(config,"%s",name);
fscanf(config,"%d",&(in->Rbitype));
fscanf(config,"%s",name);
fscanf(config,"%lf",&(in->alpha));
fscanf(config,"%s",name);
fscanf(config,"%lf",&(in->beta));

fscanf(config,"%s",name);
fscanf(config,"%d",&(in->interfaceleft));
fscanf(config,"%s",name);
fscanf(config,"%d",&(in->interfaceright));

fscanf(config,"%s",name);
fscanf(config,"%lf",&(in->phibleft));
fscanf(config,"%s",name);
fscanf(config,"%lf",&(in->phibright));

fscanf(config,"%s",name);
fscanf(config,"%lf",&(in->vl));
fscanf(config,"%s",name);
fscanf(config,"%lf",&(in->vr));

fscanf(config,"%s",name);
fscanf(config,"%d",&(in->stop_start));

fscanf(config,"%s",name);
fscanf(config,"%d",&(in->adv_sim));
if (in->adv_sim==TRUE)
{
	in->ntrapnewton=TRUE;
	in->ptrapnewton=TRUE;
}

fscanf(config,"%s",name);
fscanf(config,"%le",&(in->delta));
fscanf(config,"%s",name);
fscanf(config,"%le",&(in->gamma));

fscanf(config,"%s",name);
fscanf(config,"%d",&(in->laser));
fscanf(config,"%s",name);
fscanf(config,"%le",&(in->wavelength));
fscanf(config,"%s",name);
fscanf(config,"%le",&(in->externalv));
fclose(config);
}
