// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "sim.h"
#include <math.h>
#include <gsl/gsl_multimin.h>



struct fitvars
{
double ivdark_fitmul;
double ivlight_fitmul;
double chargedark_fitmul;
double chargelight_fitmul;
double rbi_fitmul;
double banderror_fitmul;
double muerror_fitmul;
double time_dark_mul;
double simplexmul;
int fit_rbi;
int fit_time_dark;
int fit_time_light;
int simplexreset;
int dolightfit;
int fitvars;
int dofit[40];
};

static struct fitvars fitconfig;



double get_time_error(char* ext)
{
char exp_name[20];
char sim_name[20];
char out_name[20];
sprintf(exp_name,"./chris/tpc_%s.dat",ext);
sprintf(sim_name,"./timefit_%s.dat",ext);
sprintf(out_name,"./time_error%s.dat",ext);
double error=0.0;
double max=0.0;

struct istruct exp;
struct istruct simtime;

inter_load_a(&exp,exp_name,0,1);
inter_load_a(&simtime,sim_name,0,1);

inter_log(&simtime);
inter_log(&exp);
max=inter_norm_to_one(&exp);
inter_rescale(&simtime,1.0, 1.0/max);
error=inter_get_diff(&exp,&simtime,out_name,5e-7,5e-6,fitconfig.time_dark_mul);

inter_free(&exp);
inter_free(&simtime);

return error;
}

double get_time_constant_errors()
{
double ret=0.0;
double tau=0.0;
tau=read_value("dos.inp",0,80);
ret=1+tanh((1e-14-tau)/1e-14)+1-tanh((1e-3-tau)/1e-5);

tau=read_value("dos.inp",0,82);
ret+=1+tanh((1e-14-tau)/1e-14)+1-tanh((1e-3-tau)/1e-5);

tau=read_value("dos.inp",0,84);
ret+=1+tanh((1e-14-tau)/1e-14)+1-tanh((1e-3-tau)/1e-5);

tau=read_value("dos.inp",0,86);
ret+=1+tanh((1e-14-tau)/1e-14)+1-tanh((1e-3-tau)/1e-5);

return ret;
}

double get_charge_error(char *orig_file)
{
printf("get iv error\n");
double error=0.0;
double max=0.0;
struct istruct exp;
struct istruct iv;
int fit_len=get_file_len(orig_file)-1;
int ivexternal_len=get_file_len("./charge_dark.dat")-1;
inter_load(&exp,orig_file,fit_len,1,2);
inter_load(&iv,"./charge_dark.dat",ivexternal_len,0,1);

inter_log(&iv);
inter_log(&exp);
max=inter_norm_to_one(&exp);
inter_rescale(&iv,1.0, 1.0/max);

double start=exp.x[1];
if (iv.x[0]>start) start=iv.x[0];

error=inter_get_diff(&exp,&iv,"./charge_error_dark.dat",start,0.75,fitconfig.chargedark_fitmul);

inter_free(&exp);
inter_free(&iv);

return error;
}



double get_charge_error_light(char *orig_file)
{
printf("get iv error\n");
double error=0.0;
struct istruct exp;
struct istruct iv;
double max=0.0;
int fit_len=get_file_len(orig_file)-1;
int ivexternal_len=get_file_len("./charge_light.dat")-1;
inter_load(&exp,orig_file,fit_len,1,2);
inter_load(&iv,"./charge_light.dat",ivexternal_len,0,1);
FILE *ferror;
ferror=fopen("./charge_error_light.dat","w");


inter_log(&iv);
inter_log(&exp);

max=inter_norm_to_one(&exp);
inter_rescale(&iv,1.0, 1.0/max);

double start=exp.x[1];
if (iv.x[0]>start) start=iv.x[0];
double stop=0.7;
double points=0.0;
double dx=(stop-start)/1000;
double pos=start;
double etemp=0.0;
do
{
etemp=fitconfig.chargelight_fitmul*fabs(inter_get(&iv,pos)-inter_get(&exp,pos));
fprintf(ferror,"%e %e %e %e\n",pos,etemp,inter_get(&iv,pos),inter_get(&exp,pos));
error+=etemp;
points+=1.0;

pos+=dx;
//printf("%e\n",error);
}while(pos<stop);
fclose(ferror);

inter_free(&exp);
inter_free(&iv);
//printf("return = %e\n",error);
//getchar();
return error/points;
}


double get_rbi_error()
{
char orig_file[200];
strcpy(orig_file,"./chris/k.dat");
printf("get iv error\n");
int i;
double error=0.0;
struct istruct exp;
struct istruct iv;
int fit_len=get_file_len(orig_file)-1;
int ivexternal_len=get_file_len("./rbi.dat")-1;
inter_load(&exp,orig_file,fit_len,0,1);
inter_load(&iv,"./rbi.dat",ivexternal_len,0,3);
//inter_save(&iv,"./rodtest.dat");
//getchar();
FILE *ferror;
ferror=fopen("./rbi_error.dat","w");
//inter_print(&exp);
//getchar();
//printf("len = %d %d\n", ivexternal_len,fit_len);
//getchar();
for (i=0;i<iv.len;i++)
{
if (iv.data[i]<0) iv.data[i]=0.0;
iv.data[i]=sqrt(iv.data[i]*iv.data[i]);

}
double max=0.0;
for (i=0;i<exp.len;i++)
{
//printf("%e\n",exp.data[i]);
exp.data[i]=sqrt(exp.data[i]*exp.data[i]);
if (exp.data[i]>max) max=exp.data[i];
}

for (i=0;i<iv.len;i++)
{
if (iv.data[i]>max) max=iv.data[i];
}

for (i=0;i<iv.len;i++)
{
iv.data[i]/=max;
//printf("%e\n",iv.data[i]);
}

for (i=0;i<exp.len;i++)
{
exp.data[i]/=max;
//printf("%e\n",exp.data[i]);
}
//getchar();
double start=exp.x[0];
if (iv.x[0]>start) start=iv.x[0];
double stop=exp.x[exp.len-1];
double points=0.0;
double dx=(stop-start)/1000;
double pos=start;
double etemp=0.0;
do
{
etemp=fitconfig.rbi_fitmul*fabs(inter_get(&iv,pos)-inter_get(&exp,pos));
fprintf(ferror,"%e %e %e %e\n",pos,etemp,inter_get(&iv,pos),inter_get(&exp,pos));
error+=etemp;
points+=1.0;

pos+=dx;
//printf("%e\n",error);
}while(pos<stop);

fclose(ferror);
inter_free(&exp);
inter_free(&iv);









//printf("return = %e\n",error);
//getchar();
return error/points;
}

int uptodate=FALSE;

double get_iv_error(char *orig_file)
{
double error=0.0;
struct istruct exp;
struct istruct iv;
int fit_len=get_file_len(orig_file)-1;
int ivexternal_len=get_file_len("./ivexternal_dark.dat")-1;
inter_load(&exp,orig_file,fit_len,0,2);
inter_load(&iv,"./ivexternal_dark.dat",ivexternal_len,0,1);
FILE *ferror;
ferror=fopen("./iv_error_dark.dat","w");
//inter_print(&exp);
//getchar();
//printf("len = %d %d\n", ivexternal_len,fit_len);
//getchar();
double max=0.0;

inter_log(&iv);
inter_log(&exp);
max=inter_norm_to_one(&exp);
inter_rescale(&iv,1.0, 1.0/max);

double start=exp.x[0];
if (iv.x[0]>start) start=iv.x[0];
double stop=0.6;
double points=0.0;
double dx=(stop-start)/1000;
double pos=start;
double etemp=0.0;
do
{
etemp=fitconfig.ivdark_fitmul*fabs(inter_get(&iv,pos)-inter_get(&exp,pos));
fprintf(ferror,"%e %e %e %e\n",pos,etemp,inter_get(&iv,pos),inter_get(&exp,pos));
error+=etemp;
points+=1.0;

pos+=dx;
//printf("%e\n",error);
}while(pos<stop);
fclose(ferror);
inter_free(&exp);
inter_free(&iv);
return error/points;
}

double get_band_error()
{
int dostype=read_value("dos.inp",0,2);
double Ee=0.0;
double Eh=0.0;
double ret=0.0;
if (dostype==0)
{
	Ee=read_value("dos.inp",0,38);
	Eh=read_value("dos.inp",0,40);
	ret=1+tanh( (Ee-Eh)*20.0/Ee);
	ret*=fitconfig.banderror_fitmul;
}else
{

	Ee=read_value("dos.inp",0,12);
	Eh=read_value("dos.inp",0,14);
	ret=1+tanh( (Ee-Eh)*10.0/Ee);
	ret*=fitconfig.banderror_fitmul;
}

return ret;
}

double get_mu_error()
{
int i;
double error=0.0;
struct istruct n;
struct istruct p;
int nlen=get_file_len("./munfile.dat")-1;
int plen=get_file_len("./mupfile.dat")-1;
inter_load(&n,"./munfile.dat",nlen,0,1);
inter_load(&p,"./mupfile.dat",plen,0,1);
FILE *ferror;
ferror=fopen("./mu_error.dat","w");
//inter_print(&exp);
//getchar();
//printf("len = %d %d\n", ivexternal_len,fit_len);
//getchar();

double band_error=get_band_error();
double error_tot=0.0;
double points=0.0;
for (i=0;i<p.len;i++)
{

	if ((p.x[i]>5e21)&&(p.x[i]<8e22))
	{
	error=fitconfig.muerror_fitmul*(1+tanh((p.data[i]-inter_get(&n,p.x[i]))/p.data[i]));
	error_tot+=error;
	fprintf(ferror,"%e %e %e %e %e\n",p.x[i],error,band_error,inter_get(&n,p.x[i]),p.data[i]);
	points+=1.0;
	}

}


fclose(ferror);


inter_free(&n);
inter_free(&p);
//printf("I will return %e %e %e\n",error_tot,points,band_error);
//getchar();
return error_tot/points;
}



double get_iv_error_light(char *orig_file)
{
double error=0.0;
struct istruct exp;
struct istruct iv;
inter_load_a(&exp,orig_file,0,2);
inter_load_a(&iv,"./ivexternal_light.dat",0,1);
FILE *ferror;
ferror=fopen("./iv_error_light.dat","w");

//inter_print(&exp);
//getchar();
//printf("len = %d %d\n", ivexternal_len,fit_len);
//getchar();

double max=0.0;


max=inter_norm_to_one(&exp);

inter_rescale(&iv,1.0, 1.0/max);
double start=0.4;
if (exp.x[1]>start) start=exp.x[1];
if (iv.x[0]>start) start=iv.x[0];

double stop=0.56;
double points=0.0;
double dx=(stop-start)/1000;
double pos=start;
double etemp=0.0;
do
{
etemp=fitconfig.ivlight_fitmul*fabs(inter_get(&iv,pos)-inter_get(&exp,pos));
fprintf(ferror,"%e %e %e %e\n",pos,etemp,inter_get(&iv,pos),inter_get(&exp,pos));
error+=etemp;
points+=1.0;

pos+=dx;
//printf("%e\n",error);
}while(pos<stop);

fclose(ferror);


inter_free(&exp);
inter_free(&iv);
FILE *voc_file;
double voc=0.0;
voc_file=fopen("./voc.dat","r");
fscanf(voc_file,"%le",&voc);
fclose(voc_file);
//double voc_error=fabs(0.549-voc)/2.0;
//printf("voc=%e %e %e\n",voc,error/points,voc_error);
//getchar();
return error/points;
}



double last_iv_error;
double last_charge_error;
double last_iv_error_light;
double last_charge_error_light;
double last_rbi_error;
double last_mu_error;
double last_dark_time_error;
double last_light_time_error;
double last_total_error;

double get_all_error()
{
last_iv_error=get_iv_error("./chris/dark.dat");
last_charge_error=get_charge_error("./chris/dark.dat");

if (fitconfig.dolightfit==TRUE)
{
last_iv_error_light=get_iv_error_light("./chris/1.0.dat");
last_charge_error_light=get_charge_error_light("./chris/1.0.dat");
}else
{
last_iv_error_light=0.0;
last_charge_error_light=0.0;
}

if (fitconfig.fit_time_dark==TRUE)
{
last_dark_time_error=get_time_error("0.0")+get_time_constant_errors();
}else
{
last_dark_time_error=0.0;
}

if (fitconfig.fit_time_light==TRUE)
{
last_light_time_error=get_time_error("1.0")+get_time_constant_errors();
}else
{
last_light_time_error=0.0;
}

if (fitconfig.fit_rbi==TRUE)
{
last_rbi_error=get_rbi_error();
}
else
{
last_rbi_error=0.0;
}

//printf("%e\n",last_dark_time_error);
last_mu_error=get_mu_error();

printf("last_iv_error = %e\n",last_iv_error);
printf("last_charge_error = %e\n",last_charge_error);
printf("last_iv_error_light = %e\n ",last_iv_error_light);
printf("last_charge_error_light = %e\n",last_charge_error_light);
printf("last_rbi_error = %e\n",last_rbi_error);
printf("last_dark_time_error = %e\n",last_dark_time_error);
printf("last_light_time_error = %e\n",last_light_time_error);
printf("last_mu_error = %e\n",last_mu_error);
last_total_error=last_iv_error+last_charge_error+last_rbi_error+last_iv_error_light+last_charge_error_light+last_mu_error+last_dark_time_error+last_light_time_error;
return last_total_error;
}

double my_f (const gsl_vector *v, void *params)
{
double val;
double e0;
int item=0;
int fitcount=0;

if (fitconfig.dofit[fitcount]==TRUE)
{
val = gsl_vector_get(v, item);
edit_file("blom_device.inp","",25,val);		//pcontact
item++;
}
fitcount++;

if (fitconfig.dofit[fitcount]==TRUE)
{
val = gsl_vector_get(v, item);
edit_file("blom_device.inp","",29,val);		//ncontact
item++;
}
fitcount++;

if (fitconfig.dofit[fitcount]==TRUE)
{
val = gsl_vector_get(v, item);
edit_file("blom_device.inp","",113,val);	//Dphoton
item++;
}
fitcount++;

if (fitconfig.dofit[fitcount]==TRUE)
{
val = gsl_vector_get(v, item);
edit_file("blom_device.inp","",121,val);	//alpha
item++;
}
fitcount++;

if (fitconfig.dofit[fitcount]==TRUE)
{
val = gsl_vector_get(v, item);
edit_file("blom_device.inp","",123,val);	//beta
item++;
}
fitcount++;

if (fitconfig.dofit[fitcount]==TRUE)
{
val = gsl_vector_get(v, item);
edit_file("dos.inp","",8,val);			//Ntrape
item++;
}
fitcount++;

if (fitconfig.dofit[fitcount]==TRUE)
{
val = gsl_vector_get(v, item);
edit_file("dos.inp","",10,val);			//Ntraph
item++;
}
fitcount++;

if (fitconfig.dofit[fitcount]==TRUE)
{
val = gsl_vector_get(v, item);
edit_file("dos.inp","",12,val);			//Etrape
item++;
}
fitcount++;

if (fitconfig.dofit[fitcount]==TRUE)
{
val = gsl_vector_get(v, item);
edit_file("dos.inp","",14,val);			//Etraph
item++;
}
fitcount++;

if (fitconfig.dofit[fitcount]==TRUE)
{
val = gsl_vector_get(v, item);
edit_file("dos.inp","",26,val);			//bandstarte
item++;
}
fitcount++;

if (fitconfig.dofit[fitcount]==TRUE)
{
val = gsl_vector_get(v, item);
edit_file("dos.inp","",28,val);			//bandstarth
item++;
}
fitcount++;

if (fitconfig.dofit[fitcount]==TRUE)
{
val = gsl_vector_get(v, item);
edit_file("dos.inp","",34,val);			//mue
item++;
}
fitcount++;

if (fitconfig.dofit[fitcount]==TRUE)
{
val = gsl_vector_get(v, item);
edit_file("dos.inp","",36,val);			//muh
item++;
}
fitcount++;

if (fitconfig.dofit[fitcount]==TRUE)
{
val = gsl_vector_get(v, item);
edit_file("dos.inp","",38,val);			//sigmae
item++;
}
fitcount++;

if (fitconfig.dofit[fitcount]==TRUE)
{
val = gsl_vector_get(v, item);
edit_file("dos.inp","",40,val);			//sigmah
item++;
}
fitcount++;

if (fitconfig.dofit[fitcount]==TRUE)
{
val = gsl_vector_get(v, item);
edit_file("dos.inp","",42,val);			//Ne
item++;
}
fitcount++;

if (fitconfig.dofit[fitcount]==TRUE)
{
val = gsl_vector_get(v, item);
edit_file("dos.inp","",44,val);			//Nh
item++;
}
fitcount++;

if (fitconfig.dofit[fitcount]==TRUE)
{
val = gsl_vector_get(v, item);
edit_file("blom_device.inp","",107,val);			//Nh
item++;
}
fitcount++;

if (fitconfig.dofit[fitcount]==TRUE)
{
val = gsl_vector_get(v, item);
edit_file("dos.inp","",64,val);			//Nh
item++;
}
fitcount++;

if (fitconfig.dofit[fitcount]==TRUE)
{
val = gsl_vector_get(v, item);
edit_file("dos.inp","",66,val);			//Nh
item++;
}
fitcount++;

if (fitconfig.dofit[fitcount]==TRUE)
{
val = gsl_vector_get(v, item);
edit_file("dos.inp","",68,val);			//Nh
item++;
}
fitcount++;

if (fitconfig.dofit[fitcount]==TRUE)
{
val = gsl_vector_get(v, item);
edit_file("dos.inp","",70,val);			//Nh
item++;
}
fitcount++;

if (fitconfig.dofit[fitcount]==TRUE)
{
val = gsl_vector_get(v, item);
edit_file("dos.inp","",80,val);			//taue0
item++;
}
fitcount++;

if (fitconfig.dofit[fitcount]==TRUE)
{
val = gsl_vector_get(v, item);
edit_file("dos.inp","",82,val);			//taue1
item++;
}
fitcount++;

if (fitconfig.dofit[fitcount]==TRUE)
{
val = gsl_vector_get(v, item);
edit_file("dos.inp","",84,val);			//tauh0
item++;
}
fitcount++;

if (fitconfig.dofit[fitcount]==TRUE)
{
val = gsl_vector_get(v, item);
edit_file("dos.inp","",86,val);			//tauh1
item++;
}
fitcount++;

if (fitconfig.dofit[fitcount]==TRUE)
{
val = gsl_vector_get(v, item);
edit_file("blom_device.inp","",95,val);			//laser power
item++;
}
fitcount++;


if (fitconfig.dolightfit==TRUE)
{
edit_file_int("blom_device.inp","",79,0);		//time off
edit_file("blom_device.inp","",53,1000.0);		//light on
edit_file("blom_device.inp","",57,0.3);			//change vstart
run_simulation("./blom_device.inp");
edit_file("blom_device.inp","",53,0.0);			//light off
system("cp ./ivexternal_light.dat ./ivexternal_light.old");
system("cp ./ivexternal.dat ./ivexternal_light.dat");
system("cp ./charge_light.dat ./charge_light.old");
system("cp ./charge.dat ./charge_light.dat");
}

if (fitconfig.fit_time_dark==TRUE)
{
edit_file("blom_device.inp","",53,0.0);			//light off
edit_file_int("blom_device.inp","",79,1);		//time on
run_simulation("./blom_device.inp");
edit_file_int("blom_device.inp","",79,0);		//time off

system("cp ./timefit.dat ./timefit_0.0.dat");
}

if (fitconfig.fit_time_light==TRUE)
{
edit_file("blom_device.inp","",53,1000.0);		//light on
edit_file_int("blom_device.inp","",79,1);		//time on
run_simulation("./blom_device.inp");
edit_file_int("blom_device.inp","",79,0);		//time off
edit_file("blom_device.inp","",53,0.0);			//light off

system("cp ./timefit.dat ./timefit_1.0.dat");
}

//getchar();
//printf("about to run for dark\n");
edit_file_int("blom_device.inp","",79,0);		//time off
edit_file("blom_device.inp","",53,0.0);			//light off
edit_file("blom_device.inp","",57,0.05);		//change vstart
run_simulation("./blom_device.inp");

system("cp ./ivexternal_dark.dat ./ivexternal_dark.old");
system("cp ./ivexternal.dat ./ivexternal_dark.dat");
system("cp ./charge_dark.dat ./charge_dark.old");
system("cp ./charge.dat ./charge_dark.dat");
e0=get_all_error();

return e0;
}


void fit_now()
{

FILE *out;
//out = fopen("./fitlog.dat","w");
//fclose(out);

char name[1000];
FILE *config;
config = fopen("./fit.inp","r");
if (config == NULL)
{
	printf("no file %s\n",name);
	exit(0);
}

fscanf(config,"%s",name);
fscanf(config,"%d",&(fitconfig.dolightfit));

fscanf(config,"%s",name);
fscanf(config,"%d",&(fitconfig.fit_rbi));

fscanf(config,"%s",name);
fscanf(config,"%d",&(fitconfig.fit_time_dark));

fscanf(config,"%s",name);
fscanf(config,"%d",&(fitconfig.fit_time_light));

fscanf(config,"%s",name);
fscanf(config,"%lf",&(fitconfig.time_dark_mul));

fscanf(config,"%s",name);
fscanf(config,"%lf",&(fitconfig.ivdark_fitmul));

fscanf(config,"%s",name);
fscanf(config,"%lf",&(fitconfig.ivlight_fitmul));

fscanf(config,"%s",name);
fscanf(config,"%lf",&(fitconfig.chargedark_fitmul));

fscanf(config,"%s",name);
fscanf(config,"%lf",&(fitconfig.chargelight_fitmul));

fscanf(config,"%s",name);
fscanf(config,"%lf",&(fitconfig.rbi_fitmul));

fscanf(config,"%s",name);
fscanf(config,"%lf",&(fitconfig.banderror_fitmul));

fscanf(config,"%s",name);
fscanf(config,"%lf",&(fitconfig.muerror_fitmul));

fscanf(config,"%s",name);
fscanf(config,"%lf",&(fitconfig.simplexmul));
fscanf(config,"%s",name);
fscanf(config,"%d",&(fitconfig.simplexreset));

fscanf(config,"%s",name);

int max=29;
int i;
fitconfig.fitvars=0;
for (i=0;i<max;i++)
{
fscanf(config,"%s",name);
fscanf(config,"%d",&(fitconfig.dofit[i]));
if (fitconfig.dofit[i]==TRUE) fitconfig.fitvars+=1;
printf("%d %s %d %d\n",i,name,fitconfig.fitvars,max);
}


fclose(config);
double val=0.0;
double par[5] = {1.0, 2.0, 10.0, 20.0, 30.0};

size_t iter = 0;
int status;
double size;

const gsl_multimin_fminimizer_type *T =
 gsl_multimin_fminimizer_nmsimplex;//gsl_multimin_fminimizer_nmsimplex;
gsl_multimin_fminimizer *s = NULL;
gsl_vector *ss, *x,*old_vector, *old_shift;
gsl_multimin_function minex_func;



/* Starting point */
x = gsl_vector_alloc (fitconfig.fitvars);

ss = gsl_vector_alloc (fitconfig.fitvars);

old_vector = gsl_vector_alloc (fitconfig.fitvars);

old_shift = gsl_vector_alloc (fitconfig.fitvars);

int item=0;
double shift=fitconfig.simplexmul;
int fitcount=0;
if (fitconfig.dofit[fitcount]==TRUE)
{
val=read_value("blom_device.inp",0,25);		//pcontact
gsl_vector_set (x, item, val);
gsl_vector_set (ss, item, fabs(val*shift));
item++;
}
fitcount++;

if (fitconfig.dofit[fitcount]==TRUE)
{
val=read_value("blom_device.inp",0,29);		//pcontact
gsl_vector_set (x, item, val);
gsl_vector_set (ss, item, fabs(val*shift));
item++;
}
fitcount++;


if (fitconfig.dofit[fitcount]==TRUE)
{
val=read_value("blom_device.inp",0,113);	//alpha
gsl_vector_set (x, item, val);
gsl_vector_set (ss, item, fabs(val*shift));
item++;
}
fitcount++;


if (fitconfig.dofit[fitcount]==TRUE)
{
val=read_value("blom_device.inp",0,121);	//alpha
gsl_vector_set (x, item, val);
gsl_vector_set (ss, item, fabs(val*shift));
item++;
}
fitcount++;


if (fitconfig.dofit[fitcount]==TRUE)
{
val=read_value("blom_device.inp",0,123);	//alpha
gsl_vector_set (x, item, val);
gsl_vector_set (ss, item, fabs(val*shift));
item++;
}
fitcount++;


if (fitconfig.dofit[fitcount]==TRUE)
{
val=read_value("dos.inp",0,8);			//Ntrape
gsl_vector_set (x, item, val);
gsl_vector_set (ss, item, fabs(val*shift));
item++;
}
fitcount++;


if (fitconfig.dofit[fitcount]==TRUE)
{
val=read_value("dos.inp",0,10);			//Ntraph
gsl_vector_set (x, item, val);
gsl_vector_set (ss, item, fabs(val*shift));
item++;
}
fitcount++;


if (fitconfig.dofit[fitcount]==TRUE)
{
val=read_value("dos.inp",0,12);			//Etrape
gsl_vector_set (x, item, val);
gsl_vector_set (ss, item, fabs(val*shift));
item++;
}
fitcount++;


if (fitconfig.dofit[fitcount]==TRUE)
{
val=read_value("dos.inp",0,14);			//Etraph
gsl_vector_set (x, item, val);
gsl_vector_set (ss, item, fabs(val*shift));
item++;
}
fitcount++;


if (fitconfig.dofit[fitcount]==TRUE)
{
val=read_value("dos.inp",0,26);			//bandstarte
gsl_vector_set (x, item, val);
gsl_vector_set (ss, item, fabs(val*shift));
item++;
}
fitcount++;


if (fitconfig.dofit[fitcount]==TRUE)
{
val=read_value("dos.inp",0,28);			//bandstarth
gsl_vector_set (x, item, val);
gsl_vector_set (ss, item, fabs(val*shift));
item++;
}
fitcount++;


if (fitconfig.dofit[fitcount]==TRUE)
{
val=read_value("dos.inp",0,34);			//mue
gsl_vector_set (x, item, val);
gsl_vector_set (ss, item, fabs(val*shift));
item++;
}
fitcount++;


if (fitconfig.dofit[fitcount]==TRUE)
{
val=read_value("dos.inp",0,36);			//muh
gsl_vector_set (x, item, val);
gsl_vector_set (ss, item, fabs(val*shift));
item++;
}
fitcount++;


if (fitconfig.dofit[fitcount]==TRUE)
{
val=read_value("dos.inp",0,38);			//sigmae
gsl_vector_set (x, item, val);
gsl_vector_set (ss, item, fabs(val*shift));
item++;
}
fitcount++;


if (fitconfig.dofit[fitcount]==TRUE)
{
val=read_value("dos.inp",0,40);			//sigmah
gsl_vector_set (x, item, val);
gsl_vector_set (ss, item, fabs(val*shift));
item++;
}
fitcount++;


if (fitconfig.dofit[fitcount]==TRUE)
{
val=read_value("dos.inp",0,42);			//Ne
gsl_vector_set (x, item, val);
gsl_vector_set (ss, item, fabs(val*shift));
item++;
}
fitcount++;


if (fitconfig.dofit[fitcount]==TRUE)
{
val=read_value("dos.inp",0,44);			//Nh
gsl_vector_set (x, item, val);
gsl_vector_set (ss, item, fabs(val*shift));
item++;
}
fitcount++;

if (fitconfig.dofit[fitcount]==TRUE)
{
val=read_value("blom_device.inp",0,107);	//Rshunt
gsl_vector_set (x, item, val);
gsl_vector_set (ss, item, fabs(val*shift));
item++;
}
fitcount++;

if (fitconfig.dofit[fitcount]==TRUE)
{
val=read_value("dos.inp",0,64);			//Rshunt
gsl_vector_set (x, item, val);
gsl_vector_set (ss, item, fabs(val*shift));
item++;
}
fitcount++;

if (fitconfig.dofit[fitcount]==TRUE)
{
val=read_value("dos.inp",0,66);			//Rshunt
gsl_vector_set (x, item, val);
gsl_vector_set (ss, item, fabs(val*shift));
item++;
}
fitcount++;

if (fitconfig.dofit[fitcount]==TRUE)
{
val=read_value("dos.inp",0,68);			//Rshunt
gsl_vector_set (x, item, val);
gsl_vector_set (ss, item, fabs(val*shift));
item++;
}
fitcount++;

if (fitconfig.dofit[fitcount]==TRUE)
{
val=read_value("dos.inp",0,70);			//Rshunt
gsl_vector_set (x, item, val);
gsl_vector_set (ss, item, fabs(val*shift));
item++;
}
fitcount++;

if (fitconfig.dofit[fitcount]==TRUE)
{
val=read_value("dos.inp",0,80);			//taue0
gsl_vector_set (x, item, val);
gsl_vector_set (ss, item, fabs(val*shift));
item++;
}
fitcount++;

if (fitconfig.dofit[fitcount]==TRUE)
{
val=read_value("dos.inp",0,82);			//taue1
gsl_vector_set (x, item, val);
gsl_vector_set (ss, item, fabs(val*shift));
item++;
}
fitcount++;

if (fitconfig.dofit[fitcount]==TRUE)
{
val=read_value("dos.inp",0,84);			//tauh0
gsl_vector_set (x, item, val);
gsl_vector_set (ss, item, fabs(val*shift));
item++;
}
fitcount++;

if (fitconfig.dofit[fitcount]==TRUE)
{
val=read_value("dos.inp",0,86);			//tauh1
gsl_vector_set (x, item, val);
gsl_vector_set (ss, item, fabs(val*shift));
item++;
}
fitcount++;

if (fitconfig.dofit[fitcount]==TRUE)
{
val=read_value("blom_device.inp",0,95);	//laser power
gsl_vector_set (x, item, val);
gsl_vector_set (ss, item, fabs(val*shift));
item++;
}
fitcount++;


       /* Initialize method and iterate */
       minex_func.n = item;
       minex_func.f = my_f;
       minex_func.params = par;

       s = gsl_multimin_fminimizer_alloc (T, item);
       gsl_multimin_fminimizer_set (s, &minex_func, x, ss);
     int loop=0;
       do
         {
           iter++;
           status = gsl_multimin_fminimizer_iterate(s);

           if (status)
             break;

           size = gsl_multimin_fminimizer_size (s);
           status = gsl_multimin_test_size (size, 1e-8);

           if (status == GSL_SUCCESS)
             {
               printf ("converged to minimum at\n");
             }
printf("---------------------------------------------------------------------\n");
out = fopen("./fitlog.dat","a");
gsl_vector_sub (old_vector, s->x);
gsl_vector_sub (old_shift, ss);
           printf ("%d %le %le\n", (int)iter, s->fval, size);
 		fprintf (out,"%d %le  %le ", (int)iter, s->fval, size);
		fprintf(out,"%e %e %e %e %e %e %e %e %e ",last_total_error,last_iv_error,last_charge_error,last_iv_error_light,last_charge_error_light,last_rbi_error,last_mu_error,last_dark_time_error,last_light_time_error);
		for (i=0;i<item;i++)
		{
			printf("%e %e %e %e\n",gsl_vector_get (s->x, i),gsl_vector_get(ss, i),gsl_vector_get (old_vector, i),gsl_vector_get (old_shift, i));
			fprintf(out,"%e ",gsl_vector_get (s->x, i));
		}
			printf("\n");
			fprintf(out,"\n");

fclose(out);
printf("---------------------------------------------------------------------\n");

gsl_vector_memcpy (old_vector, s->x);
gsl_vector_memcpy (old_shift, ss);
	loop++;
	if (loop>fitconfig.simplexreset)
	{
	break;
	}
         }
       while (status == GSL_CONTINUE);

       gsl_vector_free(x);
       gsl_vector_free(ss);
       gsl_multimin_fminimizer_free (s);
       gsl_vector_free (old_vector);
       gsl_vector_free (old_shift);


}

