// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "sim.h"
#include <math.h>

double sim_i(struct device *in,double wantedi)
{
double step=0.001;
double e0;
double e1;
double i0;
double i1;
double deriv;

solve_all(in,FALSE,in->external_exciton_solver);
i0=get_J_left(in)+get_J_right(in);
e0=fabs(wantedi-i0);
in->Vapplied+=step;
solve_all(in,FALSE,in->external_exciton_solver);
i1=get_J_left(in)+get_J_right(in);
e1=fabs(wantedi-i1);
deriv=(e1-e0)/step;
step=-e1/deriv;
in->Vapplied+=step;
int count=0;
int max=100;
do
{
e0=e1;
solve_all(in,FALSE,in->external_exciton_solver);
i1=get_J_left(in)+get_J_right(in);
e1=fabs(wantedi-i1);
deriv=(e1-e0)/step;
step=-e1/deriv;
double clamp=0.001;
if (e1<clamp) clamp=e1/100.0;
step=step/(1.0+fabs(step/clamp));
in->Vapplied+=step;
printf("Voc error = %e %e %e \n",in->Vapplied,e1,clamp);
if (count>max) break;
count++;
}while(e1>1e-5);

return in->Vapplied;
}

double sim_externalv(struct device *in,double wantedv)
{
double step=0.01;
double e0;
double e1;
double v0;
double v1;
double deriv;
in->Vapplied=wantedv;
solve_all(in,FALSE,in->external_exciton_solver);
v0=get_equiv_V(in);
e0=fabs(wantedv-v0);
in->Vapplied+=step;
solve_all(in,FALSE,in->external_exciton_solver);
v1=get_equiv_V(in);
e1=fabs(wantedv-v1);
deriv=(e1-e0)/step;
step=-e1/deriv;
in->Vapplied+=step;
int count=0;
int max=100;
do
{
e0=e1;
solve_all(in,FALSE,in->external_exciton_solver);
v1=get_equiv_V(in);
e1=fabs(wantedv-v1);
deriv=(e1-e0)/step;
step=-e1/deriv;
double clamp=0.001;
//if (e1<clamp) clamp=e1/100.0;
//step=step/(1.0+fabs(step/clamp));
in->Vapplied+=step;
printf("Applied = %e External = %e Error = %e\n",in->Vapplied,v1,e1);
if (count>max) break;
count++;
}while(e1>1e-5);

return in->Vapplied;
}


void sim_jv(struct device *in)
{

double V=in->Vstart;
double Vstop=in->Vstop;
double Vstep=in->Vstep;
char out[1000];
int gothermal=FALSE;
FILE *iv;
int ittr=0;
double J;
double Pden;
int first=TRUE;
double Vlast;
double Jlast;
double Pdenlast;
	do
	{

		in->Vapplied=V;



		solve_all(in,gothermal,in->external_exciton_solver);

		sprintf(out,"V=%lf current = %e mA (%e A/m^2)\n",V,get_I(in)/1e-3,J);

		printdebug(out);

//		getchar();



		if (ittr>0)
		{
			iv=fopen("./jv.dat","a");
			J=get_equiv_J(in);
			Pden=J*V;
			fprintf(iv,"%e %e %e %e %e\n",in->Vapplied,get_J(in),get_J_left(in),get_J_right(in),get_J_recom(in));
			fclose(iv);

			iv=fopen("./ivexternal.dat","a");
			//J=get_J(in);
			fprintf(iv,"%e %e\n",get_equiv_V(in),get_equiv_J(in));
			fclose(iv);


			iv=fopen("./avgmu.dat","a");
			//J=get_J(in);
			fprintf(iv,"%e %e %e\n",get_equiv_V(in),get_avg_mue(in),get_avg_muh(in));
			fclose(iv);
		}

		ittr++;
		iv=fopen("./freetrapped.dat","a");




		fprintf(iv,"%e %e %e %e %e\n",V,get_total_n_trapped_charge(in),get_total_n_charge(in),get_total_p_trapped_charge(in),get_total_p_charge(in));
		fclose(iv);


		if (V>-0.4)
		{
		iv=fopen("./charge.dat","a");
		Pden=J*V;	//+2e21
		fprintf(iv,"%e %e\n",get_equiv_V(in),get_extracted_np(in));
		fclose(iv);
		}




			plot_now(in,"plot");


			if (first==FALSE)
			{

				//check if we have crossed 0V
				if ((Vlast<=0)&&(V>=0.0))
				{
					in->Jsc=Jlast+(J-Jlast)*(0-Vlast)/(V-Vlast);
					printf("Jsc = %e\n",in->Jsc);
				}

				if ((Jlast<=0)&&(J>=0.0))
				{
					in->Voc=Vlast+(V-Vlast)*(0-Jlast)/(J-Jlast);
					printf("Voc = %e\n",in->Voc);
					FILE *voc_file;
					voc_file=fopen("./voc.dat","w");
					fprintf(voc_file,"%e\n",in->Voc);
					fclose(voc_file);
				}

				if ((Pden>Pdenlast)&&(V>0.0)&&(J<0.0))
				{
					in->Pmax=Pden;
					in->Pmax_voltage=V;
				}

			}


			Jlast=J;
			Vlast=V;
			Pdenlast=Pden;
			first=FALSE;

			V+=Vstep;
			dialog_set_progress ((in->Vstart+V)/(in->Vstop-in->Vstart));
			if ((Vstep>=0)&&(V>Vstop))
			{
				in->stop=TRUE;
				printf("--------------1\n");
			}

			if ((Vstep<0)&&(V<Vstop))
			{
				in->stop=TRUE;
				printf("--------------2\n");
			}
//printf("%e\n",Vstop);
			if (in->stop_start==TRUE) getchar();

			if (get_equiv_V(in)>Vstop) break;






		if (in->stop==TRUE)
		{
			break;

		}

	}while(1);

}
