#define BUILD_PATH "/home/roderick/solarsim/"
#ifndef sim_h
#define sim_h
#define mat_Xi		0
#define mat_Eg		1
#define mat_mue		2
#define mat_muh		3
#define mat_me		4
#define mat_mh		5
#define mat_epsilonr	6
#define mat_Dex		7
#define mat_kf		8
#define mat_a		9
#define mat_kl		10
#define mat_Xiphoton	11
#define mat_Egphoton	12
#define mat_taue0	13
#define mat_taue1	14
#define mat_tauh0	15
#define mat_tauh1	16
#define mat_A		17
#define mat_deltaE	18
#define mat_capeff      19
#define mat_br		20
#define mat_dostype	21
#define mat_rbi		22

//#define dumpconverge

#define dos_all		-1
#define dos_mob		0
#define dos_trap	2

#define dos_gau		0
#define dos_fd		1
#define dos_mb		2
#define dos_ub		3
#define dos_ub_and_gaus 4
#define dos_exp 	5
#define dos_exp_two 	7
#define dos_log 	8
#define dos_expan 	9
#define dos_expan_gaus 	10
#define dos_exp_inv 	11
#define dos_ub_and_small_gaus 12
#define dos_ub_and_small_gaus_chop 14

#define interface_schottky	 1
#ifndef TRUE
#define TRUE 1
#endif

#ifndef FALSE
#define FALSE 0
#endif

#define _quick_
#ifdef _quick_
    #define    dprintf(f,...)
#else
    #define    dprintf(f,...)    printf(f,##__VA_ARGS__)
#endif

#define epsilon0 8.85418782e-12			// m-3 kg-1 s4 A2;
#define hp 6.62606896e-34			//J S Wikipeda
#define hbar (6.62606896e-34/(2.0*M_PI))	//Calculated
#define kb 1.3806504e-23			//J K-1 Wiki
#define pi M_PI
#define q 1.602176487e-19			//C Wikipeda
#define m0 9.10938215e-31 			//Kg Wikipeda
#define cl  2.99792458e8			//m/s Wikipieda 

struct dosconfig
{
double sigma;
double edge;
double m;
int dostype;
double Nt;
double Et;
double Nt2;
double Et2;
double Eshift;
double nstart;
double nstop;
double base1;
double base2;
int npoints;
int expan_len;
double expan_N[20];
double expan_E[20];
double mu;
double tau0;
double tau1;
};

struct dos
{
int used;
double *x;
int xlen;
int tlen;
int blen;
double *t;
double *b;
double ***c;
double ***w;
double ***trap;
double **mu;
int traps;
struct dosconfig config;
};

struct mesh
{
double number;
double len;
double den;
};

struct param
{
	int used;
	int len;
	double *c;
};

struct layer
{
double height;
double Nad;
double space;
char name[100];
struct param mat[100];
};

struct layers
{
int number;
struct layer* l;
};



struct device
{
	double Jnleft;
	double Jnright;
	double Jpleft;
	double Jpright;
	int fast;
	double *phi;
	double *Nad;
	double *G;
	double *Gex;
	double **n;
	double **p;
	double **dn;
	double **dp;
	double *Eg;
	double *Xi;
	double *Ev;
	double *Ec;

	double *nfree;
	double *pfree;

	double *dnfree;
	double *dpfree;

	double **mun;
	double **mup;
	double *mun_avg;
	double *mup_avg;

	double *epsilonr;

	double **Fn;
	double **Fp;
	double *Nc;
	double *Nv;
	double *me;
	double *mh;
	double *Tl;
	double *Te;
	double *Th;
	double *ymesh;
	double *R;
	double *Fi;
	int *ymatlayers;
	double **Jn;
	double **Jp;
	double Vapplied;
	int ymeshpoints;
	double Vl;
	double Vr;
	double **x;
	double **t;
	double **xp;
	double **tp;
	double *b;
	double *ni;
	double *kf;
	double *kd;
	double *kr;
	double *a;
	double *Rbi;
	double *Rn;
	double *Rp;
	double *kl;
	double *ke;
	double *kh;
	double *Hl;
	double *He;
	double *Hh;
	double *Hasorb;
	double Psun;
	int dd_conv;
	int excite_conv;
	int thermal_conv;

	double n0;
	double deltaFln;
	double deltaFlp;
	double deltaFrn;
	double deltaFrp;

//back tracking
	double *back_x;
	double *back_xp;
	double *back_phi;
	double *back_xt;
	double *back_xpt;
	double *back_b;
	double *Rbi_k;
//back tracking end

	double *ex;
	double *Dex;
	double *Hex;

	double *nfinit;
	double *pfinit;
	double *ntinit;
	double *ptinit;

	double xlen;
	double ylen;
	double zlen;

	double **dndTe;

	double kTq;

	struct layers mat;
	struct layer lcontact;
	struct layer rcontact;
	
	int N;
	int M;
	int *Ti;	//row
	int *Tj;	//col
	double *Tx;	//data
	int enable_ex;
	int enable_plot;
	int enable_back;
	int testswitch;
//plotting
	FILE *gnuplot;
	FILE *gnuplot_time;
	FILE *converge;
	FILE *tconverge;
	int max_electrical_itt;
	double electrical_clamp;
	double thermal_clamp;
	double min_cur_error;
	double Vstart;
	double Vstop;
	double Vstep;
	int Rbi_on;
	int Gp_on;
	int Gn_on;
	int external_exciton_solver;
	int thermal_on;
	double Pmax_voltage;

//Device characterisation
	int bands;
	double Voc;
	double Jsc;
	double FF;
	double Pmax;
	double Tll;
	double Tlr;
	int Tliso;
	double dt;
	int time_on;
	int adv_sim;
	int go_time;
	double *nlast;
	double *plast;
	double *ntlast;
	double *ptlast;
	int tsteps;
	double time;
	int pulsestart;
	int pulselen;
	int reprate;
	double onvalue;
	double offvalue;
	int pulsevoltage;
	int nnewton;
	int hnewton;
	int lnewton;
	int ntrapnewton;
	int ptrapnewton;
	int combitrap;
	int Rbitype;
	int stop;
	double tau_e;
	double tau_h;
	int jouleheat;
	struct mesh *meshdata;
	int ymeshlayers;
	int onlypos;
	int odes;
	double last_error;
	double posclamp;

//n traps
	double *nt;
	double *dnt;
	double *Fnt;
	double *Nct;
	double *xt;
	double *tt;

//p traps
	double *pt;
	double *dpt;
	double *Fpt;
	double *Nvt;
	double *xpt;
	double *tpt;

	double *A;
	double *deltaE;
	double *capeff;

	double Rshunt;
	double Rcontact;
//ND filter
	double ND;
	int Dphoton;
	double Dphotoneff;
	int bdos;
	char spectra[100];

	int *dostype;
	double Vbias;

	double alpha;
	double beta;

	int do_ce;

	int interfaceleft;
	int interfaceright;
	double phibleft;
	double phibright;
	double vl;
	double vr;
	int stop_start;
	double taue0;
	double taue1;
	double tauh0;
	double tauh1;
	double delta;
	double gamma;
	int laser;
	double wavelength;
	double externalv;
};

struct istruct
{
double *x;
double *data;
int len;
};

int hashget(double *x,int N,double find);
int search(double *x,int N,double find);
void dump_for_plot(struct device *in);
void inter_load_a(struct istruct* in,char *name,int col,int ncol);
double inter_get_diff(struct istruct* one,struct istruct* two,char *filename,double start,double stop,double error_mul);
void md5write();
int md5check();
int run_simulation_remove(char *name);
void edit_file_int(char *in_name,char *front,int line,int value);
double get_dark_time_error();
double get_taue0(int type);
double get_taue1(int type);
double get_tauh0(int type);
double get_tauh1(int type);
double sim_externalv(struct device *in,double wantedv);
double sim_i(struct device *in,double wantedi);
void edit_file(char *in_name,char *front,int line,double value);
void dump_qe();
void dump_talbe();
double get_dn_w(double top,double T,double m,int type,int band);
double get_dp_w(double top,double T,double m,int type,int band);
double get_J_left(struct device *in);
double get_J_right(struct device *in);
double get_J_recom(struct device *in);
void fit_now();
int run_simulation(char *name);
double read_value(char *file, int skip,int line);
void copy_file(char *out_name, char *in_name,char *front,int line,double value);


double dos_get_Ec_edge(int i);
double dos_get_Ev_edge(int i);

void draw_gaus(struct device *in);
void sim_ce(struct device *in);
void sim_jv(struct device *in);
double carrier_count_get_rn(struct device *in);
double carrier_count_get_rp(struct device *in);

double carrier_count_get_n(struct device *in);
double carrier_count_get_p(struct device *in);
void carrier_count_reset(struct device *in);
void carrier_count_add(struct device *in);
double get_extracted_n(struct device *in);
double get_extracted_p(struct device *in);
void device_timestep(struct device *in);
void solve_all(struct device *in,int thermal,int exciton);
void sim_time(struct device *in);
double inter_norm_to_one(struct istruct* in);
void inter_log(struct istruct* in);
void inter_save(struct istruct* in,char *name);
void inter_load(struct istruct* in,char *name,int len,int col,int ncol);
double inter_get_hard(struct istruct* in,double x);
double inter_get(struct istruct* in,double x);
void inter_print(struct istruct* in);
void inter_free(struct istruct* in);
void inter_rescale(struct istruct* in,double xmul, double ymul);
void inter_mod(struct istruct* in);
void inter_norm(struct istruct* in,double mul);

double get_equiv_V(struct device *in);
double get_equiv_J(struct device *in);
void solver_free_memory(struct device *in);
void save_config(char *simfile,struct device *in);
void calculate_kd(struct device *in);
void update_heat(struct device *in);
void plot_now_excite(struct device *in);
double get_excite_error(struct device *in,double *b);
int solve_excite(struct device *in);
void black();
void device_update_photon_dist(struct device *in);
double B(double x);
double dB(double x);

double get_cur_error(struct device *in);
void update_solver_vars(struct device *in,int clamp);
void do_backtrack(struct device *in);
void fill_matrix(struct device *in);
void plot_open(struct device *in);
void solver_precon(int col,int nz,int *Ti,int *Tj, double *Tx,double *b);
void solver_dump_matrix(int col,int nz,int *Ti,int *Tj, double *Tx,double *b);
void device_init(struct device *in);
void load_config(char *simfile,struct device *in,int gui);
double get_I(struct device *in);
void update(struct device *cell);
int solve_cur(struct device *in);
int solve_cur_thermal(struct device *in,int thermal);
double mod(double in);
int solve_pos(struct device *in);
void get_initial(struct device *in);
double get_mat_param_lin(struct layer* in,int param,double x);
double get_mat_param(struct layer* in,int param);
double get_mat_param_poly(struct layer* in,int param,double x);
void mat_load_poly(FILE *file,struct param* in);
void load_mat_file(char *name,struct layer* in);
int solver(int col,int nz,int *Ti,int *Tj, double *Tx,double *b);
void solver_dump_matrix(int col,int nz,int *Ti,int *Tj, double *Tx,double *b);
void solver_get_memory(struct device *in);
void printdebug(char *text);

void backtrak_save(struct device *in);
void backtrack_restore(struct device *in);
void plot_now(struct device *in,char *name);
void plot_close(struct device *in);
void plot_replot(struct device *in);
FILE* start_gnuplot ();
double get_J(struct device *in);
void dump_thermal(struct device *in);

double get_thermal_error(struct device *in,double *b);
int solve_thermal(struct device *in);
void device_get_memory(struct device *in);
void device_free(struct device *in);
//DOS model
double get_n_den(double top,double T,double m,int type,int band);
double get_dn_den(double top,double T,double m,int type,int band);
double get_dndT_den(double top,double T,double m,int type,int band);
double get_p_den(double top,double T,double m,int type,int band);
double get_dp_den(double top,double T,double m,int type,int band);
double get_dpdT_den(double top,double T,double m,int type,int band);
double get_n_trap_den(double top,double T,double m,int type,int band);
double get_p_trap_den(double top,double T,double m,int type,int band);
double get_n_mu(double top,double T,double m,int type,int timedomain);
double get_p_mu(double top,double T,double m,int type,int timedomain);
double get_dn_trap_den(double top,double T,double m,int type,int band);
double get_dp_trap_den(double top,double T,double m,int type, int band);
void gen_dos_maxwell_boltzmann();
void gen_dos_fd();
void load_dos(struct device *dev,char *namen, char *namep,int type);
void gen_dos_fd_gaus_fd();
void gen_dos_maxwell_boltzmann();
double get_n_w(double top,double T,double m,int type,int band);
double get_p_w(double top,double T,double m,int type,int band);
double get_top_from_n(double n,double T,double m,double Nc,int type,int band);
double get_top_from_p(double p,double T,double m,double Nc,int type,int band);
void dos_init();
void dos_free();
//end of DOS model
double get_charge(struct device *in);
int get_file_len(char *file_name);
double get_mat_param_mobility(struct layer* in,int param,int n);
double get_mat_param_xy(struct layer* in,int param,double x);
void mat_free(struct layer* in);
void dos_set_cap_rate(double capeff);
double get_extracted_np(struct device *in);
double get_extracted_k(struct device *in);

void light_field(struct device *in,char *name,double Plaser);
void rbi_test(struct device *in);
void dirty_fit();
double get_mat_param_dmobility(struct layer* in,int param,double n);

double get_bi_error(int ymeshpoints,double *b);
void solve_bi();

double get_avg_mue(struct device *in);
double get_avg_muh(struct device *in);


double get_total_n_charge(struct device *in);
double get_total_p_charge(struct device *in);
double get_total_n_trapped_charge(struct device *in);
double get_total_p_trapped_charge(struct device *in);
void solver_free();

#endif

