// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "sim.h"
#include <math.h>


void sim_time(struct device *in)
{
FILE *train;
FILE *iv;

carrier_count_reset(in);

in->go_time=FALSE;

train=fopen("./train.dat","w");
fclose(train);

train=fopen("./cej.dat","w");
fclose(train);

train=fopen("./cer.dat","w");
fclose(train);

train=fopen("./cetogo.dat","w");
fclose(train);

train=fopen("./cemu.dat","w");
fclose(train);

train=fopen("./timefit.dat","w");
fclose(train);

train=fopen("./iv.dat","w");
fclose(train);


printf("\n\n ----------------- \n\n");
double V=in->Vstart;
int ittr=0;
double total_steps=0;
int gothermal=FALSE;
int step=0;
double J;
double Pden;
char out[1000];

light_field(in,in->spectra,0.0);
sim_externalv(in,in->externalv);
device_timestep(in);
double Ifirst=fabs(get_I(in));
//printf("%e %e\n",V,in->Vapplied);
//getchar();
in->go_time=TRUE;
V=in->Vapplied;

if (in->pulsevoltage==TRUE)
{
V=in->onvalue;
}
int turnofflight=TRUE;
in->time=-in->pulsestart*in->dt;

do
{
//printf("%d %d\n",(step>=in->pulsestart),((in->pulsestart+in->pulselen)>step));
//getchar();
	if ((step>=in->pulsestart)&&(step<(in->pulsestart+in->pulselen)))
	{
		if (in->pulsevoltage==TRUE)
		{
			V=in->onvalue;
		}else
		{
			if (in->laser==FALSE)
			{
				in->Psun=in->onvalue;
				light_field(in,in->spectra,0.0);
			}else
			{
				light_field(in,in->spectra,in->onvalue);
			}
		}
	}else
	{
if (in->pulsevoltage==TRUE)
		{
			V=in->offvalue;
			if (turnofflight==TRUE)
			{
				in->Psun=0;
				light_field(in,in->spectra,0.0);
				turnofflight=FALSE;
				if (in->testswitch==TRUE) in->Rbi_on=FALSE;
			}
		}else
		{
			if (in->laser==FALSE)
			{
				in->Psun=in->offvalue;
				light_field(in,in->spectra,0.0);
			}else
			{
				light_field(in,in->spectra,in->offvalue);
			}
		}
		//printf("triggered1 %d %d\n",(step>=in->pulsestart),((in->pulsestart+in->pulselen)>step));
		//getchar();
		//printf("off!\n");
		//getchar();
		//printf("triggered2 %d %d\n",(step>=in->pulsestart),((in->pulsestart+in->pulselen)>step));
		//getchar();
	}




	in->Vapplied=V;

	//if (step>0)
	//{
	//	in->go_time=TRUE;
	//}

	printf("time=%e step=%d %.1e ",in->time,step,in->last_error);

	solve_all(in,gothermal,in->external_exciton_solver);

	if (V==0) carrier_count_add(in);

	sprintf(out,"V=%lf current = %e mA (%e A/m^2)\n",V,get_I(in)/1e-3,get_J(in));

	printdebug(out);


	iv=fopen("./iv.dat","a");
	J=get_J(in);
	Pden=J*V;
	fprintf(iv,"%e %e %e\n",in->Vapplied,get_I(in),J);
	fclose(iv);

	iv=fopen("./avgmu.dat","a");
	//J=get_J(&cell);
	fprintf(iv,"%e %e %e\n",get_equiv_V(in),get_avg_mue(in),get_avg_muh(in));
	fclose(iv);


	ittr++;
	iv=fopen("./freetrapped.dat","a");
	fprintf(iv,"%e %e %e\n",get_equiv_V(in),get_total_n_trapped_charge(in)/get_total_n_charge(in),get_total_p_trapped_charge(in)/get_total_p_charge(in));
	fclose(iv);

	plot_now(in,"plot");


	train=fopen("./train.dat","a");
	fprintf(train,"%.20e %.20e %.20e %.20e %.20e %.20e\n",in->time,in->Vapplied,in->Psun,get_J(in),get_extracted_n(in),get_extracted_p(in));
	fclose(train);

	train=fopen("./cetogo.dat","a");
	fprintf(train,"%.20e %.20e %.20e\n",in->time,get_extracted_n(in),get_extracted_p(in));
	fclose(train);

	train=fopen("./cej.dat","a");
	fprintf(train,"%.20e %.20e %.20e\n",in->time,carrier_count_get_n(in),carrier_count_get_p(in));
	fclose(train);

	train=fopen("./cer.dat","a");
	fprintf(train,"%.20e %.20e %.20e\n",in->time,carrier_count_get_rn(in),carrier_count_get_rp(in));
	fclose(train);

	train=fopen("./cemu.dat","a");
	fprintf(train,"%.20e %.20e %.20e\n",in->time,get_avg_mue(in),get_avg_muh(in));
	fclose(train);

	train=fopen("./timefit.dat","a");
	fprintf(train,"%.20e %.20e\n",in->time,fabs(get_I(in))-Ifirst);
	fclose(train);

	device_timestep(in);

	step++;

	if (step>in->reprate)
	{
		step=0;
		total_steps+=1.0;
	}



	dialog_set_progress ((total_steps)/(in->tsteps));

	if (total_steps==in->tsteps) break;
//printf("%lf %d\n",total_steps,in->tsteps);

	if (in->stop==TRUE)
	{
		break;

	}

if (in->stop_start==TRUE) getchar();
}while(1);

in->go_time=FALSE;

}

void device_timestep(struct device *in)
{
int i;
int band=0;

for (i=0;i<in->ymeshpoints;i++)
{
	in->nlast[i]=in->n[i][band];
	in->plast[i]=in->p[i][band];
	in->ntlast[i]=in->nt[i];
	in->ptlast[i]=in->pt[i];
}
in->time+=in->dt;
}

